#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
CCMetagen main script

@ V.R.Marcelino
Created on Wed Jul 25 17:13:10 2018

"""

version_numb = 'v1.2.0'

# imports
import sys
import pandas as pd
from argparse import ArgumentParser
import subprocess
from ete3 import NCBITaxa
import re

# local imports
from ccmetagen import fParseKMA
from ccmetagen import cTaxInfo # needed in fParseKMA
from ccmetagen import fNCBItax # needed in fParseKMA

# help
if len(sys.argv) == 1:
    print ("")
    print ("CCMetagen - Identify species in metagenome datasets")
    print (version_numb)
    print ("To be used with KMA")
    print ("")
    print ("Usage: CCMetagen.py <options> ")
    print ("Ex: CCMetagen.py -i KMA_out/2_mtg.res -o 2_mtg_result")
    print ("")
    print ("")
    print ("""When running CCMetagen on multiple files in a folder:
input_dir=KMA_out
output_dir=CCMetagen_results
mkdir $output_dir
for f in $input_dir/*.res; do
    out=$output_dir/${f/$input_dir\/}
    CCMetagen.py -i $f -o $out
done""")
    print ("")
    print ("For help and options, type: CCMetagen.py -h")
    print ("")
    print ("")
    sys.exit()


parser = ArgumentParser()

parser.add_argument('-m', '--mode', default = 'both',
                    help="""what do you want CCMetagen to do?
                    Valid options are 'visual', 'text' or 'both':
                        text: parses kma, filters based on quality and output a text file with taxonomic information and detailed mapping information
                        visual: parses kma, filters based on quality and output a simplified text file and a krona html file for visualization
                        both: outputs both text and visual file formats. Default = both""", required=False)

parser.add_argument('-i', '--res_fp', help='Path to the KMA result (.res file)', required=True)
parser.add_argument('-o', '--output_fp', default = 'CCMetagen_out',
                    help='Path to the output file. Default = CCMetagen_out', required=False)
parser.add_argument('-r', '--reference_database', default = 'nt',
                    help='Which reference database was used. Options: UNITE, RefSeq or nt. Default = nt', required=False)
parser.add_argument('-ef', '--extended_output_file', default = 'n',
                    help="""Produce an extended output file that includes the percentage of classified reads.
                    Options: y or n. To use this featire, you need to generate the mapstat file when
                    required unning KMA (use flag -ef), and use it as input in CCMetagen (flag --mapstat). 
                    Default = n""", required=False)

parser.add_argument('-du', '--depth_unit', default = 'kma',
                    help="""Desired unit for Depth(abundance) measurements.
                    Default = kma (KMA default depth, which is the number of nucleotides overlapping each template,
                    divided by the lengh of the template).
                    Alternatively, you can have abundance calculated in Reads Per Million (RPM, option 'rpm'), or 
                    simply count the number of nucleotides overlaping the template (option 'nc').
                    If you use the 'nc' or 'rpm' options, remember to change the default --depth parameter accordingly.
                    Valid options are nc, rpm and kma""", required=False)
parser.add_argument('-map', '--mapstat', help="""Path to the mapstat file produced with KMA when using the -ef flag (.mapstat).
                    Required when calculating abundances in RPM or when producing the extended_output_file""", required = False)
parser.add_argument('-d', '--depth', default = 0.2,
                    help="""minimum sequencing depth. Default = 0.2.
                    If you use --depth_unit nc, change this accordingly. For example, -d 200 (200 nucleotides) 
                    is similar to -d 0.2 when using the default '--depth_unit kma' option.""",type=float, required=False)
parser.add_argument('-c', '--coverage', default = 20,
                    help="""Minimum coverage. Default = 20 (i.e. 20%% of the reference sequence)""",type=float, required=False)
parser.add_argument('-q', '--query_identity', default = 50,
                    help='Minimum query identity (Phylum level). Default = 50', type=float, required=False)
parser.add_argument('-p', '--pvalue', default = 0.05,
                    help='Minimum p-value. Default = 0.05.',type=float, required=False)

# similarity thresholds:
parser.add_argument('-st', '--species_threshold', default = 98.41,
                    help='Species-level similarity threshold. Default = 98.41',type=float, required=False)
parser.add_argument('-gt', '--genus_threshold', default = 96.31,
                    help='Genus-level similarity threshold. Default = 96.31',type=float, required=False)
parser.add_argument('-ft', '--family_threshold', default = 88.51,
                    help='Family-level similarity threshold. Default = 88.51',type=float, required=False)
parser.add_argument('-ot', '--order_threshold', default = 81.21,
                    help='Order-level similarity threshold. Default = 81.21',type=float, required=False)
parser.add_argument('-ct', '--class_threshold', default = 80.91,
                    help='Class-level similarity threshold. Default = 80.91',type=float, required=False)
parser.add_argument('-pt', '--phylum_threshold', default = 0,
                    help='Phylum-level similarity threshold. Default = 0 - not applied',type=float, required=False)
parser.add_argument('-off', '--turn_off_sim_thresholds', default = 'n',
                    help='Turns simularity-based filtering off. Options = y or n. Default = n', required=False)

parser.add_argument('--version', action='version', version=version_numb)

args = parser.parse_args()

mode = args.mode
f = args.res_fp
ref_database = args.reference_database
c = args.coverage
q = args.query_identity
du = args.depth_unit
d = args.depth
p = args.pvalue
mapstat = args.mapstat
ef = args.extended_output_file

# taxononomic thresholds:
off = args.turn_off_sim_thresholds

if off == 'y':
    st = 0
    gt = 0
    ft = 0
    ot = 0
    ct = 0
    pt = 0

elif off == 'n':
    st = args.species_threshold
    gt = args.genus_threshold
    ft = args.family_threshold
    ot = args.order_threshold
    ct = args.class_threshold
    pt = args.phylum_threshold

else:
    print ("-off argument should be either y or n. ")
    sys.exit("Try again.")

# developing and debugging:
#args.output_fp = "CCMetagen_nt_results"
#f = "sim_metagen.res"
#mapstat="sim_metagen.mapstat" # make a way of finding this automatically?? Flag -mapstat?
#ref_database = "nt"
#mode = 'text'
#c = 20
#q = 50
#d = 0.2
#p = 0.05
#st = 99
#gt = 98
#ft = 95
#ot = 80
#ct = 0
#pt = 0
#du = 'kma'
#ef = 'y' # extended format - add a flag, default = 'n'


##### Checks:

# Run implicitly ete3.NCBITaxa.__init__() to check for valid taxonomy database
NCBITaxa()

# Warning if RefDatabase is unknown
if ref_database not in ("UNITE", "RefSeq","nt"):
    print (""" Reference database (-r) must be either UNITE, RefSeq or nt.
           the input is case sensitive and the default is nt.""")
    sys.exit("Try again.")


# check if ef flag is correct
if ef not in ("y", "n"):
    print ("Unrecognized argument %s. Use either '-ef n' (default) or '-ef y'" %(ef))
    sys.exit("Try again.")


##### Read input files and output a pandas dataframe
print ("")
print ("Reading file %s" %(f))
print ("")

df = pd.read_csv(f, sep='\t', index_col=0, encoding='latin1')

# Rename headers:
df.index.name = "Closest_match"


##### Adjust depth to reflect number of bases or RPM if needed:

# number of nucleotides:
if du == 'nc':
    df['Depth'] = df.Depth * df.Template_length
    print ("Calculating depth as number of nucleotides, ignoring template length.")
    print ("""Remember to adjust minimum depth value (ex: -d 200) to filter low abundance hits.""")

# RPM:   
elif du == 'rpm':
    print ("Calculating RPM...")
    print ("""
           Note 1: to calculate RPM, you need to generate the mapstat file when
           running KMA (flag -ef), and use it as input in CCMetagen (flag --mapstat).
           
           Note 2: you might want to adjust the minimum depth (-d) value accordingly.
           The default minimum depth is 0.2.
           """)

    with open(mapstat) as mapfile:
        fragments_line=mapfile.readlines()[3]
    total_frags = re.split(r'(\t|\n)',fragments_line)[2]
    df_stats = pd.read_csv(mapstat, sep='\t', index_col=0, header = 6, encoding='latin1')
    df['Depth'] = 1000000 * df_stats['fragmentCount'] / int(total_frags)

elif du == 'kma':
    print ("")

else:
    print ("""Warning: the depth unit you specified makes no sense.
           --depth_unit option must be nc, rpm, or kma. Using 'kma'.""")
    print ("")

##### Quality control + taxonomic assignments
    
# quality filter (coverage, query identity, Depth and p-value)
df = fParseKMA.res_filter(df, ref_database, c, q, d, p)

# add tax info
df = fParseKMA.populate_w_tax(df, ref_database, st, gt, ft, ot, ct, pt)


##### Output a file with tax info
if (mode == 'text') or (mode == 'both'):

    # save to file
    out = args.output_fp + ".csv"
    pd.DataFrame.to_csv(df, out)

    print ("csv file saved as %s" %(out))
    print ("")

##### Output a Krona file
if (mode == 'visual') or (mode == 'both'):
    krona_info = df[['Depth','Superkingdom','Kingdom','Phylum','Class','Order','Family','Genus','Species']]

    # remove the unk_xx for better krona representation
    krona_info = krona_info.replace('unk_.*$', value = '',regex=True)

    # save dataframe to file
    out1 = args.output_fp + ".tsv"
    pd.DataFrame.to_csv(krona_info, out1, sep='\t', index=False, header=False)

    # save krona file
    out2 = args.output_fp + ".html"

    shell_command = "ktImportText " + out1 + " -o " + out2
    subprocess.run(shell_command, shell=True)

    print ("krona file saved as %s" %(out2))
    print ("")
    

##### Extended format - calculate read mapping stats
if ef == 'y':
    print ("calculating read mapping stats...")
           
    with open(mapstat) as mapfile:
        fragments_line=mapfile.readlines()[3]
    total_frags = re.split(r'(\t|\n)',fragments_line)[2]
    df_stats = pd.read_csv(mapstat, sep='\t', index_col=0, header = 6, encoding='latin1')
    
    # delete species in df_stats that are not in the CCM result dataframe:
    df_stats_filt = df_stats[df_stats.index.isin(df.index)].copy() # the copy handles the pandas warning
    df_stats_filt['perc_map'] = df_stats_filt['fragmentCount'] / int(total_frags) * 100
    total_mapped = sum(df_stats_filt['perc_map'])
    
    stats_out = args.output_fp + "_stats.csv"
    pd.DataFrame.to_csv(df_stats_filt, stats_out )
    
    print ("\nStats file saved as %s" %(stats_out))
    print ("""\nProportion of reads mapped to the database: %f%%\n""" %(total_mapped))
    






    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Classes that store taxonomic info from matches and NCBI lineage

@ V.R.Marcelino
Created on 12 Jul 2018
Last update on 27 March 2019

"""

### Stores taxid and other info
class TaxInfo(): 
    
    def __init__(self, TaxId=None, Lineage=None, Sample=None, RefDatabase=None, 
                 Abundance=None, Superkingdom = None, Superkingdom_TaxId = None, 
                 Kingdom=None, Kingdom_TaxId=None, Phylum=None, 
                 Phylum_TaxId=None, Class=None, Class_TaxId=None,
                 Order=None, Order_TaxId=None, Family=None, Family_TaxId=None,
                 Genus=None, Genus_TaxId=None, Species=None, Species_TaxId=None,
                 LCA_TaxId=None, Coverage=None):
        
        # info from matches
        self.TaxId = TaxId
        self.Lineage = Lineage # the results of the match, not considering taxIDs
        self.Sample = Sample
        self.RefDatabase = RefDatabase
        self.Abundance = Abundance

        # info from NCBI
        self.Superkingdom = Superkingdom
        self.Superkingdom_TaxId = Superkingdom_TaxId
        self.Kingdom = Kingdom
        self.Kingdom_TaxId = Kingdom_TaxId
        self.Phylum = Phylum
        self.Phylum_TaxId = Phylum_TaxId
        self.Class = Class
        self.Class_TaxId = Class_TaxId
        self.Order = Order
        self.Order_TaxId = Order_TaxId
        self.Family = Order
        self.Family_TaxId = Order_TaxId
        self.Genus = Genus
        self.Genus_TaxId = Genus_TaxId
        self.Species = Species
        self.Species_TaxId = Species_TaxId
        
        # Taxid of the Lowest Common Ancestor
        self.LCA_TaxId = LCA_TaxId

        # info from Fungi Table only
        self.Coverage = Coverage



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
CCMetagen_merge.py

Merge the abundance results of CCMetagen for several samples into a single table.
The results must be in .csv format (default CCMetagen or ---mode text)
And no other csv file should be present in the folder

USAGE example 1: Merge table at family level:
CCMetagen_merge.py -i CCMetagen_folder -t Family

USAGE example 2: Merge table by species (default) and keep only Cryptococcus and Candida:
CCMetagen_merge.py -i CCMetagen_folder -kr k -l Genus -tlist Candida,Cryptococcus

@ V.R.Marcelino
Created on Tue Dec 18 10:00:48 2018
"""

import sys
import pandas as pd 
from argparse import ArgumentParser
import os


parser = ArgumentParser()
parser.add_argument('-i', '--input_fp', help="""Path to the folder containing CCMetagen text results.
                    Note that files must end with ".csv"
                    and the folder should not contain other .csv files""", required=True)
parser.add_argument('-t', '--tax_level', default = 'Species',
                    help="""Taxonomic level to merge the results. Options:
                    Closest_match (includes different genes for the same species),
                    Species (Default), Genus, Family, Order, Class, Phylum, Kingdom and Superkingdom
                    """, required=False)
parser.add_argument('-o', '--output_fp', default = 'merged_samples', 
                    help='Path to the output file. Default = merged_samples', required=False)

# arguments to filter by taxonomy:
parser.add_argument('-kr', '--keep_or_remove', default = 'n', 
                    help='keep or remove taxa. Options = k (keep), r (remove) and n (none, default)', required=False)

parser.add_argument('-l', '--filtering_tax_level', default = 'none', 
                    help='level to perform taxonomic filtering, default = none', required=False)

parser.add_argument('-tlist', '--taxa_list', default = [], type=str, 
                    help='list taxon names (comma-separated) that you want to keep or exclude', required=False)

args = parser.parse_args()
in_folder = args.input_fp
tax_level = args.tax_level
output = args.output_fp
kr = args.keep_or_remove


# debugging:
#in_folder = "CCM_res_nt"
#tax_level = "Species"
#output = "merged_samples_depth_species_no_ecoli"
#kr = "r" # k for keep, r for remove, and n for none
#level = "Species"
#taxa = ["Escherichia coli"]


# Set the taxonomic levels to retain info:
l = ['Superkingdom','Kingdom','Phylum','Class','Order','Family','Genus','Species','Closest_match']

f4agg = {}

f4agg['Depth']= 'sum'
for i in l:
    if i == tax_level:
        f4agg[i] = 'first'
        break
    else:
        f4agg[i] = 'first'
    
# list of wanted taxa (no depth):
wanted_taxa = list(f4agg.keys())
wanted_taxa.remove('Depth')

# create new dataframe:
all_samples = pd.DataFrame()


# read input files and merge results
for file in os.listdir(in_folder):
    if file.endswith(".csv"):
        
        sample_name = file.split(".csv")[0]
        result_fp = os.path.join(in_folder, file)

        df = pd.read_csv(result_fp, sep=',', index_col=0)
        
        # this is needed because groupby excludes rows with NAs 
        df = df.fillna("NA")
        
        # keep or remove taxa:
        if kr == "k":
            level = args.filtering_tax_level
            taxa = args.taxa_list.split(",")
            filt_df_str = "df[df['{}'].isin({})]".format(level,taxa)
            df = eval(filt_df_str)
            
        elif kr == "r":
            level = args.filtering_tax_level
            taxa = args.taxa_list.split(",")
            filt_df_str = "df[~df['{}'].isin({})]".format(level,taxa)
            df = eval(filt_df_str)
        elif kr == "n":
            pass        
        else:
            print ("kr must be k (keep), r (remove) or n (none).")
            sys.exit("Try again.")
 
        # if tax_level = closest_match, we need an extra column:
        
        if tax_level == 'Closest_match':
            df['Closest_match'] = df.index
            df.index.name = None # This is needed in modern versions of Pandas (1.0.1)

        depth_by_tax = df.groupby(by=wanted_taxa).agg(f4agg)
        depth_by_tax.rename(columns={'Depth':sample_name}, inplace=True)

        all_samples = pd.concat([all_samples, depth_by_tax], sort=True, axis=1)
    

# Group taxon ranks 
all_samples = all_samples.groupby(by=all_samples.columns, axis = 1).first()

# add taxon info at the end of the table:
tax_cols_l = list(f4agg.keys())
tax_cols_l.remove('Depth')

tax_cols = all_samples[tax_cols_l]
sample_cols = all_samples.drop(columns=tax_cols_l)

all_samples = pd.merge(sample_cols,tax_cols,left_index=True,right_index=True)

# Fill NaN with zeros:
all_samples = all_samples.fillna(0)

# Remove the artificially added NAs:
all_samples = all_samples.replace(["NA"], value = "")


### Save
out = output + ".csv"
pd.DataFrame.to_csv(all_samples, out, index=False)

print ("Done. Abundance estimates for all samples saved as %s" %(out))





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Function that takes a taxid as input and returns a class containing the taxonomic info and taxids
for multiple taxonomic ranks.

Required to parse the results of KMA and store them in the SQLite3 'bench.db'

@ V.R.Marcelino
Created on 12 Jul 2018

"""

from ete3 import NCBITaxa
from ccmetagen import cTaxInfo  # where we define classes used here


def lineage_extractor(query_taxid, TaxInfo_object):
    list_of_taxa_ranks = ['superkingdom', 'kingdom', 'phylum', 'class', 'order', 'family','genus', 'species']
    ncbi = NCBITaxa()
    lineage = ncbi.get_lineage(query_taxid)
    ranks = ncbi.get_rank(lineage)
    names = ncbi.get_taxid_translator(lineage)

# get known data

    for key, val in ranks.items():

        if val == list_of_taxa_ranks[0]:
            TaxInfo_object.Superkingdom = names[key]
            TaxInfo_object.Superkingdom_TaxId = key

        elif val == list_of_taxa_ranks[1]:
            TaxInfo_object.Kingdom = names[key]
            TaxInfo_object.Kingdom_TaxId = key

        elif val == list_of_taxa_ranks[2]:
            TaxInfo_object.Phylum = names[key]
            TaxInfo_object.Phylum_TaxId = key

        elif val == list_of_taxa_ranks[3]:
            TaxInfo_object.Class = names[key]
            TaxInfo_object.Class_TaxId = key

        elif val == list_of_taxa_ranks[4]:
            TaxInfo_object.Order = names[key]
            TaxInfo_object.Order_TaxId = key

        elif val == list_of_taxa_ranks[5]:
            TaxInfo_object.Family = names[key]
            TaxInfo_object.Family_TaxId = key

        elif val == list_of_taxa_ranks[6]:
            TaxInfo_object.Genus = names[key]
            TaxInfo_object.Genus_TaxId = key

        elif val == list_of_taxa_ranks[7]:
            TaxInfo_object.Species = names[key]
            TaxInfo_object.Species_TaxId = key

# fill in the blanks
    if TaxInfo_object.Superkingdom is None:
        TaxInfo_object.Superkingdom = "unk_sk"

    if TaxInfo_object.Kingdom is None:
        TaxInfo_object.Kingdom = "unk_k"

    if TaxInfo_object.Phylum is None:
        TaxInfo_object.Phylum = "unk_p"

    if TaxInfo_object.Class is None:
        TaxInfo_object.Class = "unk_c"

    if TaxInfo_object.Order is None:
        TaxInfo_object.Order = "unk_o"

    if TaxInfo_object.Family is None:
        TaxInfo_object.Family = "unk_f"

    if TaxInfo_object.Genus is None:
        TaxInfo_object.Genus = "unk_g"

    if TaxInfo_object.Species is None:
        TaxInfo_object.Species = "unk_s"


    return TaxInfo_object



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script to extract sequence reads (or contigs) assigned to a given taxon

Created on Wed Jun 10 19:37:36 2020

@author: V.R.Marcelino
"""


#imports

import sys
import pandas as pd
import csv
from argparse import ArgumentParser


# help
if len(sys.argv) == 1:
    print ("")
    print ("""Produces a fasta file containing the sequences assigned to a given taxon""")   
    print ("For help and options, type: CCMetagen_extract_seqs.py -h")
    print ("")


parser = ArgumentParser()
parser.add_argument('-ifrag', '--input_frags', help='The path to the file containing frags (decompressed)', required=True)
parser.add_argument('-iccm', '--input_ccmetagen', help='The path to the ccmetagen result csv file', required=True)
parser.add_argument('-l', '--taxonomic_level', help="""Taxonomic level to merge the results. Options:
                    Species, Genus, Family, Order, Class, Phylum, Kingdom and Superkingdom""", required=True)
parser.add_argument('-t', '--taxon', help="""Taxon for which you want to extract sequences. 
                    Use quotation marks to specify species (e.g. -t 'Escherichia coli')""", required=True)
parser.add_argument('-o', '--output_fp', default = 'wanted_taxon_seqs', 
                    help='Path to the output file. Default = wanted_taxon_seqs', required=False)


args = parser.parse_args()
frags_fp  = args.input_frags
iccm = args.input_ccmetagen
tax_level = args.taxonomic_level
tax_name = args.taxon
o_file = args.output_fp + ".fas"


# developing and debugging:
#frags_fp = "test2.frag"
#iccm = "/test2_CCMetagen_nt_results.csv"
#tax_name = "Aspergillaceae"
#tax_level = "Family"
#o_file = "wanted_seq.fas"

## Get a list for all Closest_match where <tax_rank> == <tax_name>:
df = pd.read_csv(iccm, sep=',', index_col=0)

wanted_df = df[df[tax_level].str.match(tax_name).fillna(False)]


# Then extract sequences for that taxon.
ofile_good = open(o_file, "w")

cg = 0 #count good reads
with open(frags_fp) as frags:
    for line in csv.reader(frags, delimiter = '\t'):
        match = line[5]
        seq_id = line[6]
        seq_read = line[0]

        
        if match in wanted_df.index.values:
            ofile_good.write(">"+ seq_id + "\n" + seq_read + "\n")
            cg += 1  

ofile_good.close()

print ("")
print ("Done.")
print ("%i sequences classified as %s were saved in the %s file." %(cg, tax_name, o_file))
print ("Forward and reverse reads were saved to the same file")
print ("")





#!/usr/bin/env ipython
# -*- coding: utf-8 -*-
"""
Functions to parse KMA results

@ V.R.Marcelino
Created on 1 Aug 2018


"""
import re

# local imports
from ccmetagen import cTaxInfo
from ccmetagen import fNCBItax

import subprocess

# function to filter a res file in pandas df format:
def res_filter(df,ref_database, cov,Iden,Depth,p):
    df = df.drop(df[df.Template_Coverage < cov].index)

    # filter based on identity
    df = df.drop(df[df.Query_Identity < Iden].index)

    # filter based on depth
    df = df.drop(df[df.Depth < Depth].index)

    # filter based on p-values
    df = df.drop(df[df.p_value > p].index)

    return df


# function that takes as input a pandas dataframe with KMA results
# and add tax information to results
def populate_w_tax(in_df, ref_database,species_threshold,genus_threshold,
                   family_threshold,order_threshold,class_threshold,phylum_threshold):
    #defaults:
    #species_threshold = 98.41 # Yeast - Vu et al 2016
    #genus_threshold = 96.31 # Yeast - Vu et al 2016
    #family_threshold = 88.51 # Filamentous fungi - Vu et al 2019
    #order_threshold = 81.21 # Filamentous fungi - Vu et al 2019
    #class_threshold = 80.91 # Filamentous fungi - Vu et al 2019
    #phylum, kingdom and superkingdom = 0  # no data, no filtering


    # Make sure all taxa columns are strings (doesn't automatically happen if the first one is None)
    in_df = in_df.assign(LCA_TaxId="",Superkingdom="",Kingdom="",Phylum="",Class="",Order="",Family="",Genus="",Species="")


    # index == the #template (fungal match)
    for index, row in in_df.iterrows():
        match_info = cTaxInfo.TaxInfo()

        # define the tax. rank based on similarity:
        if ref_database == "UNITE":
            split_match = re.split (r'(\|| )', index)
            qiden = row['Query_Identity']
            match_info.Lineage = split_match[12]

            # if taxid is knwon:
            if split_match[4] != 'unk_taxid':

                match_info.TaxId = int(split_match[4])
                match_info = fNCBItax.lineage_extractor(match_info.TaxId , match_info)

                # Warning about unknown taxids:
            else:
                print ("")
                print ("WARNING: based on accession number, no taxonomic information was found in NCBI for %s" %(match_info.Lineage))
                print ("This match will not get NCBItax taxonomic ranks")
                print ("")
                match_info.TaxId = split_match[4] # 'unk_taxid'


        elif ref_database == "RefSeq":
            split_match = re.split (r'(\|| )', index)
            qiden = row['Query_Identity']
            match_info.TaxId = int(split_match[4])
            species = split_match[6] + " " + split_match[8]
            match_info.Lineage = species
            # include info from NCBI:
            match_info = fNCBItax.lineage_extractor(match_info.TaxId, match_info)


        elif ref_database == "nt":
            split_match = re.split (r'(\|| )', index)
            qiden = row['Query_Identity']
            match_info.Lineage = split_match[2]

            #get taxid from accession number
            taxid = split_match[0]

            if taxid == 'unk_taxid':
                # Warning about unknown taxids:
                print ("")
                print ("WARNING: no NCBI's taxid found for accession %s" %(match_info.Lineage))
                print ("This match will not get taxonomic ranks")
                print ("")

            else:
                match_info.TaxId = int(taxid)
                match_info = fNCBItax.lineage_extractor(match_info.TaxId, match_info)


        # Populate the df with lineage info and the LCA taxid:

        in_df.at[index, 'Superkingdom'] = match_info.Superkingdom
        in_df.at[index, 'Kingdom'] = match_info.Kingdom

        # Assign LCA_taxid. Go to Kingdom if possible:
        in_df.at[index, 'LCA_TaxId'] = match_info.Superkingdom_TaxId

        if match_info.Kingdom_TaxId is not None:
            in_df.at[index, 'LCA_TaxId'] = match_info.Kingdom_TaxId

        # if it matches to uncultured or unclassified fungus, use the Fungi LCA itaxid:
        if match_info.Kingdom == 'Fungi':
            in_df.at[index, 'LCA_TaxId'] = 4751

        # fill in the rest of the table according to similarity threshold:
        if qiden >= phylum_threshold:
            in_df.at[index, 'Phylum'] = match_info.Phylum
            if match_info.Phylum_TaxId != None:
                in_df.at[index, 'LCA_TaxId'] = match_info.Phylum_TaxId

        if qiden >= class_threshold:
            in_df.at[index, 'Class'] = match_info.Class
            if match_info.Class_TaxId != None:
                in_df.at[index, 'LCA_TaxId'] = match_info.Class_TaxId

        if qiden >= order_threshold:
            in_df.at[index, 'Order'] = match_info.Order
            if match_info.Order_TaxId != None:
                in_df.at[index, 'LCA_TaxId'] = match_info.Order_TaxId

        if qiden >= family_threshold:
            in_df.at[index, 'Family'] = match_info.Family
            if match_info.Family_TaxId != None:
                in_df.at[index, 'LCA_TaxId'] = match_info.Family_TaxId

        if qiden >= genus_threshold:
            in_df.at[index, 'Genus'] = match_info.Genus
            if match_info.Genus_TaxId != None:
                in_df.at[index, 'LCA_TaxId'] = match_info.Genus_TaxId

        if qiden >= species_threshold:
            in_df.at[index, 'Species'] = match_info.Species
            if match_info.Species_TaxId != None:
                in_df.at[index, 'LCA_TaxId'] = match_info.Species_TaxId

    return in_df



